@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Inline;
 
-our $DATE = '2015-06-19'; # DATE
-our $VERSION = '0.10'; # VERSION
+our $DATE = '2015-07-02'; # DATE
+our $VERSION = '0.11'; # VERSION
 
 use 5.010001;
 use strict;
@@ -42,6 +42,7 @@ my %pericmd_attrs = (
     )} qw/actions common_opts completion default_subcommand get_subcommand_from_arg
           description exit formats
           riap_client riap_version riap_client_args
+          log
           subcommands
           tags
           read_env env_name
@@ -61,12 +62,26 @@ my %pericmd_attrs = (
         summary => 'Program name',
         schema => 'str*',
     },
+    program_version => {
+        summary => 'Program version (otherwise will use version from url metadata)',
+        schema => 'str',
+    },
     url => {
         summary => 'Program URL',
         schema => 'str*',
         pos => 0,
         'x.schema.entity' => 'riap_url',
     },
+    extra_urls_for_version => {
+        summary => 'More URLs to show version for --version',
+        description => <<'_',
+
+Currently not implemented in Perinci::CmdLine::Inline.
+
+_
+        schema => ['array*', of=>'str*'],
+        'x.schema.element_entity' => 'riap_url',
+    },
 );
 
 $SPEC{gen_inline_pericmd_script} = {
@@ -179,6 +194,11 @@ _
             'x.schema.element_entity' => 'modulename',
         },
 
+        code_before_parse_cmdline_options => {
+            schema => 'str*',
+            tags => ['category:extra-code'],
+        },
+
         output_file => {
             summary => 'Set output file, defaults to stdout',
             schema => 'str*',
@@ -591,8 +611,9 @@ _
                     } elsif ($specmeta->{common_opt} eq 'version') {
                         no strict 'refs';
                         push @l, '        print "', $program_name , ' version ',
-                            ($mod && ${"$mod\::VERSION"} ? ${"$mod\::VERSION"} : '?'),
-                            ($mod && ${"$mod\::DATE"} ? " (".${"$mod\::DATE"}.")" : '?'),
+                            (defined($args{program_version}) ? $args{program_version} :
+                             $mod && ${"$mod\::VERSION"} ? ${"$mod\::VERSION"} : '?'),
+                            ($mod && ${"$mod\::DATE"} ? " (".${"$mod\::DATE"}.")" : ''),
                             '\n";', "\n";
                         push @l, '        print "  Generated by ', __PACKAGE__ , ' version ',
                             (${__PACKAGE__."::VERSION"} // 'dev'),
@@ -622,7 +643,7 @@ _
                             push @l, '$_pci_args{', $specmeta->{arg}, '} = $_[1];';
                         }
                     } else {
-                        if ($specmeta->{parsed}{type} =~ /\@/) {
+                        if (($specmeta->{parsed}{type} // '') =~ /\@/) {
                             push @l, 'if ($mentioned_args{', $specmeta->{arg}, '}++) { push @{ $_pci_args{', $specmeta->{arg}, '} }, $_[1] } else { $_pci_args{', $specmeta->{arg}, '} = [$_[1]] }';
                         } else {
                             push @l, '$_pci_args{', $specmeta->{arg}, '} = $_[1];';
@@ -700,6 +721,11 @@ _
             scalar(localtime), ".\n",
             "# You probably should not manually edit this file.\n\n",
 
+            # for dzil
+            "# DATE\n",
+            "# VERSION\n",
+            "\n",
+
             $dp_code1,
 
             "package main;\n",
@@ -719,6 +745,8 @@ _
                 sort keys %{$cd->{subs}}),
             "\n",
 
+            ("# code before parse cmdline options \n\n", $args{code_before_parse_cmdline_options}) x !!$args{code_before_parse_cmdline_options},
+
             @l,
 
             $dp_code2,
@@ -773,7 +801,7 @@ Perinci::CmdLine::Inline - Generate inline Perinci::CmdLine CLI script
 
 =head1 VERSION
 
-This document describes version 0.10 of Perinci::CmdLine::Inline (from Perl distribution Perinci-CmdLine-Inline), released on 2015-06-19.
+This document describes version 0.11 of Perinci::CmdLine::Inline (from Perl distribution Perinci-CmdLine-Inline), released on 2015-07-02.
 
 =head1 SYNOPSIS
 
@@ -799,23 +827,23 @@ B<EARLY DEVELOPMENT.>
 
 Sample startup overhead benchmark vs Perinci::CmdLine::Lite:
 
-                                   Rate  run 1 (lite) run --version (lite)   -c (lite) run 1 (inline) run --version (inline) -c (inline)   perl
- run 1 (lite)             13.59+-0.02/s            --               -61.5%      -80.8%         -87.8%                 -90.0%      -92.0% -97.5%
- run --version (lite)   35.286+-0.053/s 159.65+-0.54%                   --      -50.2%         -68.3%                 -74.0%      -79.3% -93.6%
- -c (lite)                70.92+-0.83/s   421.9+-6.1%            101+-2.4%          --         -36.3%                 -47.8%      -58.4% -87.1%
- run 1 (inline)           111.31+-0.2/s   719.1+-1.9%        215.46+-0.73%    57+-1.8%             --                 -18.1%      -34.7% -79.8%
- run --version (inline)   135.9+-0.36/s       900+-3%          285.1+-1.2%  91.6+-2.3%   22.09+-0.39%                     --      -20.2% -75.4%
- -c (inline)               170.4+-2.9/s     1154+-21%          382.9+-8.1% 140.3+-4.9%     53.1+-2.6%             25.4+-2.1%          -- -69.1%
- perl                      551.3+-1.9/s     3957+-15%         1462.4+-5.9% 677.4+-9.5%    395.3+-1.9%            305.7+-1.8% 223.5+-5.5%     --
+                                   Rate  run 1 (lite) run --version (lite)   -c (lite) run --version (inline) run 1 (inline) -c (inline)   perl
+ run 1 (lite)           13.715+-0.017/s            --               -61.2%      -81.4%                 -88.2%         -88.9%      -90.7% -97.2%
+ run --version (lite)   35.355+-0.032/s 157.78+-0.39%                   --      -52.2%                 -69.6%         -71.4%      -76.1% -92.8%
+ -c (lite)                73.89+-0.46/s   438.7+-3.4%            109+-1.3%          --                 -36.4%         -40.2%      -50.1% -84.9%
+ run --version (inline)    116.3+-3.8/s      748+-28%             229+-11%  57.4+-5.3%                     --          -5.9%      -21.5% -76.3%
+ run 1 (inline)          123.51+-0.58/s   800.5+-4.4%          249.3+-1.7%  67.2+-1.3%              6.2+-3.5%             --      -16.7% -74.8%
+ -c (inline)               148.2+-2.3/s      981+-17%          319.2+-6.6% 100.6+-3.4%             27.5+-4.6%         20+-2%          -- -69.7%
+ perl                    489.67+-0.97/s  3470.3+-8.3%             1285+-3% 562.7+-4.3%               321+-14%      296.5+-2% 230.4+-5.3%     --
  
  Average times:
-   perl                  :     1.8139ms
-   -c (inline)           :     5.8685ms
-   run --version (inline):     7.3584ms
-   run 1 (inline)        :     8.9839ms
-   -c (lite)             :    14.1004ms
-   run --version (lite)  :    28.3399ms
-   run 1 (lite)          :    73.5835ms
+   perl                  :     2.0422ms
+   -c (inline)           :     6.7476ms
+   run 1 (inline)        :     8.0965ms
+   run --version (inline):     8.5985ms
+   -c (lite)             :    13.5336ms
+   run --version (lite)  :    28.2845ms
+   run 1 (lite)          :    72.9129ms
 
 =head1 FUNCTIONS
 
@@ -883,6 +911,8 @@ Arguments ('*' denotes required arguments):
 
 Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
 
+=item * B<code_before_parse_cmdline_options> => I<str>
+
 =item * B<common_opts> => I<any>
 
 Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
@@ -915,6 +945,12 @@ Currently does nothing, provided only for compatibility with Perinci::CmdLine::B
 
 Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
 
+=item * B<extra_urls_for_version> => I<array[str]>
+
+More URLs to show version for --version.
+
+Currently not implemented in Perinci::CmdLine::Inline.
+
 =item * B<formats> => I<any>
 
 Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
@@ -927,6 +963,10 @@ Currently does nothing, provided only for compatibility with Perinci::CmdLine::B
 
 Include extra modules.
 
+=item * B<log> => I<any>
+
+Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
+
 =item * B<meta> => I<hash>
 
 An alternative to specifying `url`.
@@ -947,6 +987,10 @@ Whether to pass Perinci::CmdLine::Inline object.
 
 Program name.
 
+=item * B<program_version> => I<str>
+
+Program version (otherwise will use version from url metadata).
+
 =item * B<read_config> => I<any>
 
 Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
